from django.db import models


class Blacklist(models.Model):
    TYPE_CHOICES = (
        (0, 'Bounce'),
        (1, 'Complaints'),
    )
    email = models.EmailField(unique=True)
    type = models.PositiveSmallIntegerField(default=0, choices=TYPE_CHOICES)
    created_at = models.DateTimeField(auto_now_add=True)

    def __unicode__(self):
        return self.email


RESULT_CODES = (
    ("1", "success"),
    ("2", "retry"),
    ("3", "blacklisted"),
)

class MessageLogManager(models.Manager):
    def log(self, message, result_code):
        self.create(email=message.to[0], subject=message.subject, result=result_code)
        

class MessageLog(models.Model):
    email = models.EmailField()
    subject = models.CharField(max_length=255)
    result = models.CharField(max_length=1, choices=RESULT_CODES)
    created_at = models.DateTimeField(auto_now_add=True)

    objects = MessageLogManager()

    def __unicode__(self):
        return self.email



from django.core.mail.backends.base import BaseEmailBackend
from django.conf import settings

from .tasks import send_email


class CeleryEmailBackend(BaseEmailBackend):
    def __init__(self, fail_silently=False, **kwargs):
        super(CeleryEmailBackend, self).__init__(fail_silently)
        self.init_kwargs = kwargs

    def send_messages(self, email_messages, **kwargs):
        results = []
        kwargs['_backend_init_kwargs'] = self.init_kwargs
        for msg in email_messages:
            NO_DELAY = getattr(settings, 'DJCELERY_SES_NO_DELAY', False)
            if NO_DELAY:
                result = send_email(msg, **kwargs)
            else:
                result = send_email.delay(msg, **kwargs)
            results.append(result)
        return results



import os.path

from django.test import TestCase
from django.core import mail
from django.core.mail import EmailMessage
from django.test.utils import override_settings

from .models import Blacklist
from .utils import pass_blacklist, no_delay


@override_settings(
    EMAIL_BACKEND='djcelery_ses.backends.CeleryEmailBackend',
    CELERY_EMAIL_BACKEND='django.core.mail.backends.locmem.EmailBackend',
)
class DjcelerySESTest(TestCase):
    def test_send_mail(self):
        msg = EmailMessage('title', 'body content', 'noreply@example.com', ['noreply@example.com'])
        msg.send()

        self.assertEqual(len(mail.outbox), 1)

    def test_blacklist(self):
        # Add `noreply@example.com` to Blacklist
        Blacklist.objects.create(email='noreply@example.com', type=0)
        
        # Send email to `noreply@example.com`
        msg = EmailMessage('title', 'body content', 'noreply@example.com', ['noreply@example.com'])
        msg.send()

        # should be no email in outbox
        self.assertEqual(len(mail.outbox), 0)

    def test_pass_blacklist(self):
         # Add `noreply@example.com` to Blacklist
        Blacklist.objects.create(email='noreply@example.com', type=0)
        
        # Send email to `noreply@example.com`
        with pass_blacklist:
            msg = EmailMessage('title', 'body content', 'noreply@example.com', ['noreply@example.com'])
            msg.send()

        # should be one email in outbox
        self.assertEqual(len(mail.outbox), 1)

    def test_no_delay(self):
        with no_delay: 
            msg = EmailMessage('title', 'body content', 'noreply@example.com', ['noreply@example.com'])
            msg.send()
            
        self.assertEqual(len(mail.outbox), 1)

class SNSNotificationTest(TestCase):
    urls = 'djcelery_ses.urls'

    def test_notification(self):
        PROJECT_ROOT = os.path.dirname(os.path.realpath(__file__))
        FIXTURE_DIRS = os.path.join(PROJECT_ROOT, 'fixtures')

        with open(os.path.join(FIXTURE_DIRS, 'sns.json')) as f:
            content = f.read()

        self.client.post('/sns_notification/', content, content_type="application/json")

        self.assertEqual(Blacklist.objects.count(), 1)

    def test_error_notification(self):
        resp = self.client.post('/sns_notification/', 'hello', content_type="application/json")
        self.assertEqual(resp.content.decode(), 'Invalid JSON')
        self.assertEqual(resp.status_code, 400)


    def test_subscription(self):
        PROJECT_ROOT = os.path.dirname(os.path.realpath(__file__))
        FIXTURE_DIRS = os.path.join(PROJECT_ROOT, 'fixtures')

        with open(os.path.join(FIXTURE_DIRS, 'subscription.json')) as f:
            content = f.read()

        self.client.post('/sns_notification/', content, content_type="application/json")




# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Deleting field 'MessageLog.body'
        db.delete_column(u'djcelery_ses_messagelog', 'body')

        # Adding field 'MessageLog.subject'
        db.add_column(u'djcelery_ses_messagelog', 'subject',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=255),
                      keep_default=False)


    def backwards(self, orm):
        # Adding field 'MessageLog.body'
        db.add_column(u'djcelery_ses_messagelog', 'body',
                      self.gf('django.db.models.fields.TextField')(default=''),
                      keep_default=False)

        # Deleting field 'MessageLog.subject'
        db.delete_column(u'djcelery_ses_messagelog', 'subject')


    models = {
        u'djcelery_ses.blacklist': {
            'Meta': {'object_name': 'Blacklist'},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '75'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'type': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'})
        },
        u'djcelery_ses.messagelog': {
            'Meta': {'object_name': 'MessageLog'},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'result': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['djcelery_ses']


# coding: utf-8
import json

from django.http import HttpResponse, HttpResponseBadRequest
from django.views.decorators.csrf import csrf_exempt
from django.core.mail import mail_admins

from .models import Blacklist


@csrf_exempt
def sns_notification(request):
    """
    Receive AWS SES bounce SNS notification
    """

    # decode json
    try:
        data = json.loads(request.read())
    except ValueError:
        return HttpResponseBadRequest('Invalid JSON')

    # handle SNS subscription
    if data['Type'] == 'SubscriptionConfirmation':
        subscribe_url = data['SubscribeURL']
        subscribe_body = """
        Please visit this URL below to confirm your subscription with SNS

        %s """ % subscribe_url

        mail_admins('Please confirm SNS subscription', subscribe_body)
        return HttpResponse('OK')

    #
    try:
        message = json.loads(data['Message'])
    except ValueError:
        assert False, data['Message']

    #
    type = 0 if message['notificationType'] == 'Bounce' else 1
    email = message['mail']['destination'][0]


    # add email to blacklist
    try:
        Blacklist.objects.get(email=email)
    except Blacklist.DoesNotExist:
        Blacklist.objects.create(email=email, type=type)

    return HttpResponse('Done')



from django.contrib import admin
from .models import Blacklist, MessageLog

class BlacklistAdmin(admin.ModelAdmin):
    list_display = ('email', 'type', 'created_at')
    list_filter = ('type',)
admin.site.register(Blacklist, BlacklistAdmin)


class MessageLogAdmin(admin.ModelAdmin):
    list_display = ('email', 'result', 'created_at')
admin.site.register(MessageLog, MessageLogAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Blacklist',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(unique=True, max_length=75)),
                ('type', models.PositiveSmallIntegerField(default=0, choices=[(0, b'Bounce'), (1, b'Complaints')])),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MessageLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(max_length=75)),
                ('subject', models.CharField(max_length=255)),
                ('result', models.CharField(max_length=1, choices=[(b'1', b'success'), (b'2', b'retry'), (b'3', b'blacklisted')])),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.conf import settings

class PassBlacklist(object):
    def __enter__(self):
        settings.DJCELERY_SES_CHECK_BLACKLIST = False

    def __exit__(self, type, value, tb):
        settings.DJCELERY_SES_CHECK_BLACKLIST = True

pass_blacklist = PassBlacklist()


class NoDelay(object):
    def __enter__(self):
        settings.DJCELERY_SES_NO_DELAY = True

    def __exit__(self, type, value, tb):
        settings.DJCELERY_SES_NO_DELAY = False

no_delay = NoDelay()



from smtplib import SMTPDataError

from django.conf import settings
from django.db import IntegrityError
from django.core.mail import get_connection

from celery.task import task

from .models import Blacklist, MessageLog


CONFIG = getattr(settings, 'CELERY_EMAIL_TASK_CONFIG', {})
BACKEND = getattr(settings, 'CELERY_EMAIL_BACKEND',
                  'django.core.mail.backends.smtp.EmailBackend')

TASK_CONFIG = {
    'name': 'djcelery_email_send',
    'ignore_result': True,
}
TASK_CONFIG.update(CONFIG)


@task(**TASK_CONFIG)
def send_email(message, **kwargs):
    """
    send mail task
    """

    logger = send_email.get_logger()
    conn = get_connection(backend=BACKEND)

    # check blacklist
    CHECK_BLACKLIST = getattr(settings, 'DJCELERY_SES_CHECK_BLACKLIST', True)
    if CHECK_BLACKLIST:
        logger.debug('Check blacklist')

        try:
            Blacklist.objects.get(email=message.to[0], type=0)
            logger.debug("Email already in blacklist.")
            return
        except Blacklist.DoesNotExist:
            pass

    # send
    try:
        result = conn.send_messages([message])
        logger.debug("Successfully sent email message to %r.", message.to)
        MessageLog.objects.log(message, 1)
        return result
    except SMTPDataError as e:
        logger.warning("Message to %r, blacklisted.", message.to)
        if e.smtp_code == 554:
            MessageLog.objects.log(message, 3)
            try:
                Blacklist(email=message.to[0]).save()
            except IntegrityError:
                pass
    except Exception as e:
        MessageLog.objects.log(message, 2)
        logger.warning("Failed to send email message to %r, retrying.", message.to)
        send_email.retry(exc=e)

    conn.close()



from setuptools import setup, find_packages
from djcelery_ses import __version__


setup(
    name='django-celery-ses',
    version=__version__,
    description="django-celery-ses",
    author='tzangms',
    author_email='tzangms@streetvoice.com',
    url='http://github.com/StreetVoice/django-celery-ses',
    license='MIT',
    test_suite='runtests.runtests',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "django >= 1.3, < 1.9",
        "django-celery >= 3",
    ],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Framework :: Django',
        'Environment :: Web Environment',
    ],
    keywords='django,celery,mail',
)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Blacklist'
        db.create_table(u'djcelery_ses_blacklist', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('email', self.gf('django.db.models.fields.EmailField')(unique=True, max_length=75)),
            ('type', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'djcelery_ses', ['Blacklist'])

        # Adding model 'MessageLog'
        db.create_table(u'djcelery_ses_messagelog', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('body', self.gf('django.db.models.fields.TextField')()),
            ('result', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'djcelery_ses', ['MessageLog'])


    def backwards(self, orm):
        # Deleting model 'Blacklist'
        db.delete_table(u'djcelery_ses_blacklist')

        # Deleting model 'MessageLog'
        db.delete_table(u'djcelery_ses_messagelog')


    models = {
        u'djcelery_ses.blacklist': {
            'Meta': {'object_name': 'Blacklist'},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '75'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'type': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'})
        },
        u'djcelery_ses.messagelog': {
            'Meta': {'object_name': 'MessageLog'},
            'body': ('django.db.models.fields.TextField', [], {}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'result': ('django.db.models.fields.CharField', [], {'max_length': '1'})
        }
    }

    complete_apps = ['djcelery_ses']